@@ -85,7 +85,7 @@ and the main function is to make them be methods of the same object, e.g.:
  this work and send me a patch!
 """
 
-import cProfile, operator, time
+import operator, time
 from decimal import Decimal as D
 
 #from pyutil import jsonutil as json
@@ -104,24 +104,48 @@ def makeg(func):
             func()
     return blah
 
-def rep_bench(func, n, initfunc=None, MAXREPS=10, MAXTIME=60.0, profile=False, profresults="pyutil-benchutil.prof", UNITS_PER_SECOND=1):
+def to_decimal(x):
+    """
+    See if D(x) returns something. If instead it raises TypeError, x must have been a float, so convert it to Decimal by way of string. (In Python >= 2.7, D(x) does this automatically.
+    """
+    try:
+        return D(x)
+    except TypeError:
+        return D("%0.54f" % (x,))
+
+def mult(a, b):
+    """
+    If we get TypeError from * (possibly because one is float and the other is Decimal), then promote them both to Decimal.
+    """
+    try:
+        return a * b
+    except TypeError:
+        return to_decimal(a) * to_decimal(b)
+
+def rep_bench(func, n, initfunc=None, MAXREPS=10, MAXTIME=60.0, profile=False, profresultsfileobj=None, UNITS_PER_SECOND=1, quiet=False):
     """
     Will run the func up to MAXREPS times, but won't start a new run if MAXTIME
     (wall-clock time) has already elapsed (unless MAXTIME is None).
+
+    @param quiet Don't print anything--just return the results dict.
     """
     assert isinstance(n, int), (n, type(n))
+
+    if profile and profresultsfileobj is None:
+        raise Exception('If you ask for profiling, you are required to pass a profresultsfileobject to write the results into.')
+
     startwallclocktime = time.time()
-    tls = [] # elapsed time in nanoseconds
+    tls = [] # elapsed time in seconds
     bmes = []
     while ((len(tls) < MAXREPS) or (MAXREPS is None)) and ((MAXTIME is None) or ((time.time() - startwallclocktime) < MAXTIME)):
         if initfunc:
             initfunc(n)
         try:
-            tl = bench_it(func, n, profile=profile, profresults=profresults)
+            tl = bench_it(func, n, profile=profile, profresultsfileobj=profresultsfileobj)
         except BadMeasure, bme:
             bmes.append(bme)
         else:
-            tls.append(tl * UNITS_PER_SECOND)
+            tls.append(tl)
     if len(tls) == 0:
         raise Exception("Couldn't get any measurements within time limits or number-of-attempts limits. Maybe something is wrong with your clock? %s" % (bmes,))
     sumtls = reduce(operator.__add__, tls)
@@ -142,17 +166,21 @@ def rep_bench(func, n, initfunc=None, MAXREPS=10, MAXTIME=60.0, profile=False, p
     # etc, so we use mp1 to name it.
     mp1 = m+1
     res = {
-        'worst': worst/n,
-        'best': best/n,
+        'worst': mult(worst, UNITS_PER_SECOND)/n,
+        'best': mult(best, UNITS_PER_SECOND)/n,
         'mp1': mp1,
-        'mth-best': mthbest/n,
-        'mth-worst': mthworst/n,
-        'mean': mean/n,
+        'mth-best': mult(mthbest, UNITS_PER_SECOND)/n,
+        'mth-worst': mult(mthworst, UNITS_PER_SECOND)/n,
+        'mean': mult(mean, UNITS_PER_SECOND)/n,
         'num': len(tls),
         }
 
-    print "best: %(best)#8.03e, %(mp1)3dth-best: %(mth-best)#8.03e, mean: %(mean)#8.03e, %(mp1)3dth-worst: %(mth-worst)#8.03e, worst: %(worst)#8.03e (of %(num)6d)" % res
+    if not quiet:
+        print "best: %(best)#8.03e, %(mp1)3dth-best: %(mth-best)#8.03e, mean: %(mean)#8.03e, %(mp1)3dth-worst: %(mth-worst)#8.03e, worst: %(worst)#8.03e (of %(num)6d)" % res
 
+    if profile:
+        import statprof
+        statprof.display(profresultsfileobj)
     return res
 
 MARGINOFERROR = 10
@@ -175,15 +203,22 @@ class BadMeasure(Exception):
 def do_nothing(n):
     pass
 
-def bench_it(func, n, profile=False, profresults="pyutil-benchutil.prof"):
+def bench_it(func, n, profile=False, profresultsfileobj=None):
     if profile:
+        import statprof
         st = clock()
-        cProfile.run('func(n)', profresults)
-        sto = clock()
+        statprof.start()
+        try:
+            func(n)
+        finally:
+            statprof.stop()
+            sto = clock()
     else:
         st = clock()
-        func(n)
-        sto = clock()
+        try:
+            func(n)
+        finally:
+            sto = clock()
     timeelapsed = sto - st
     if timeelapsed <= 0:
         raise BadMeasure(timeelapsed)
